package flare.vis
{
	import flare.animate.Transitioner;
	import flare.data.DataSet;
	import flare.vis.axis.Axes;
	import flare.vis.axis.CartesianAxes;
	import flare.vis.controls.PanZoomControl;
	import flare.vis.controls.TooltipControl;
	import flare.vis.data.Data;
	import flare.vis.data.EdgeMode;
	import flare.vis.data.NodeMode;
	import flare.vis.data.render.EdgeRenderer;
	import flare.vis.data.render.IRenderer;
	import flare.vis.data.skins.EdgeSkin;
	import flare.vis.events.TooltipEvent;
	import flare.vis.operator.Operator;
	import flare.vis.operator.layout.Layout;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import mx.binding.utils.BindingUtils;
	import mx.collections.ArrayCollection;
	import mx.core.IDataRenderer;
	import mx.core.IToolTip;
	import mx.core.UIComponent;
	import mx.events.FlexEvent;
	import mx.utils.object_proxy;
	
	/**
	 * <b>FlareVisualization</b>: The Flex component you include in your projects. Usually you will extend this class
	 * and define your <code>nodeSkin</code> and <code>layout</code> in your constructor as a minimum (please do call super(data)).
	 * <p>
	 * You will then need to define your <code>buildData</code> function to process your particular dataset into a collection
	 * of nodes and edges. Bind the <code>rawData</code> variable to your arraycollection of data and it'll be called whenever 
	 * the data changes.
	 * </p>
	 * <p>
	 * Normal usage is of form:<br>
	 * <code> &lt;vis:FlareVisualisation rawData="{dataArr}" 
						   width="500" height="500" / &gt;</code>
	 * </p>
	 * @author FlareForFlex is an extension and reworking of the Flare library (http://flare.prefuse.org/) (which itself was a reworking of
	 * the prefuse library for java). 
	 * Flare was released with a BSD license, and this honours that by being released with a FreeBSD license:
	 * <p>
	 * Copyright (c) 2012, Simon O'Connor (simon(at)oconnorlamb.com), and original Flare contributors (2007),  All rights reserved.
	 * </p>
	 * <p>
	 * Redistribution and use in source and binary forms, with or without
	 * modification, are permitted provided that the following conditions are met: 
	 * </p>
	 * <p>
	 * 1. Redistributions of source code must retain the above copyright notice, this
   	 * list of conditions and the following disclaimer.</p>
  	 * <p>
	 * 2. Redistributions in binary form must reproduce the above copyright notice,
   	 * this list of conditions and the following disclaimer in the documentation
   	 * and/or other materials provided with the distribution. 
	 * </p>
 	 * <p>
	 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
	 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	 * </p>
 	 * <p>
	 * The views and conclusions contained in the software and documentation are those
	 * of the authors and should not be interpreted as representing official policies, 
	 * either expressed or implied, of the FreeBSD Project.
	 * </p>  
	 */
	public class FlareVisualization extends UIComponent implements IDataRenderer
	{
		// ========================================
		// Config properties
		// ========================================	
		
	
		/**
		 * Skin to apply to nodes within this visualisation. They must  
		 */		
		public var nodeSkin:Class;
		
		/**
		 * Specify here to change the renderer used for edges, usually leave it alone as it handles the 
		 * drawing of the lines
		 */		
		public var edgeRenderer:IRenderer = EdgeRenderer.instance;
		
		
		/**
		 * Edges will still need to render using the renderer method, as it is really
		 * a drawing process rather than a skin...  but you can still apply a skin to display
		 * other things on an edge if you wish. This defaults to an empty skin.
		 */		
		public var edgeSkin:Class = EdgeSkin;	
		
		
		/**
		 * If you want the toolTip arg on the nodes to be set to
		 * the contents of a field of the data, specify the fieldname here 
		 */
		public var tooltipField:String;
				
				
		/**
		 * Set this to shift the visualization within its container. This allows you to 
		 * have parts of the visualisation not be cropped by the boundaries 
		 */
		public var xOffset:Number = 15;
		
		
		/**
		 * Set this to shift the visualization within its container. This allows you to 
		 * have parts of the visualisation not be cropped by the boundaries 
		 */
		public var yOffset:Number = 15;
		
		/**
		 * Do you want mouse pan and zooming enabled by default? This will 
		 * created a PanZoomControl if left as true. 
		 */
		public function get panZoomEnabled():Boolean{
			return _panZoomEnabled;			
		}
		public function set panZoomEnabled(b:Boolean):void{
			//trace('SET PAN AND ZOOM ='+b +' VIS='+visualization);
			_panZoomEnabled = b;
			
			
			//If no controls and no pan and zoom required, just leave
			if(!_controls && !b){
				return;
			}
			
			//If we need a pan and zoom and no control array, make one.
			if(b && !_controls){
				_controls = new Array();
			}
						
			for(var i:int=0;i<_controls.length;i++){				
				var c:Object = _controls[i];				
				if(c is PanZoomControl){					
					if(b) return; //If set to true, but we already have one... just leave
					else{//otherwise we must remove it.						
						delete _controls[i];
						controls = _controls;
						return;
					}
				}
			}
			
			if(b && visualization){ //No existing PZ found, one needed, but can only add if we have the vis in order to set hit area
				var pc:PanZoomControl = new PanZoomControl(visualization);
				pc.zoomEnabled = true;					
				_controls.push(pc);
				controls = _controls;
			}					
		}
		private var _panZoomEnabled:Boolean = true;
		
		/**
		 * On many visualisations you want to have different line styles and start and end shapes based 
		 * on what the types of nodes are on each end. This gives you a way to define that in a quick way.
		 * 
		 * Use in this form:
		 * <p>edgeStyling = 	{ 'view_table:data_movement':{dashedLine:false},
								'*:view_table':{dashedLine:true,dashLength:2,dashGapLength:6,endShapeHeight:7,
												endShapeWidth:6,endShapeType:LineEndType.OPENARROW},
								'file:*':{dashedLine:true,dashLength:5,dashGapLength:3},
								'db_server:*':{dashedLine:true,dashLength:5,dashGapLength:3}								
							};</p>
		 * <p>{FROMNODETYPE:TONODETYPE:{KEYVALUE PAIRS OF EDGE ATTRIBUTES}}</p>
		 * An asterisk can be used to denote any node type on that side of the colon. Asterisks on both sides
		 * denote the default properties for all edges
		 */
		public var edgeStyling:Object;
		
		/**
		 * By default this will resize itself to be the size of its visualisation (so you
		 * can have scrollbars etc. But sometimes you want it to stay fixed and use its own controls
		 * for moving around the vis. true = size to the vis, false = height/width set here.
		 */		
		public function get autoResize():Boolean{
			return _autoResize;
		}
		public function set autoResize(b:Boolean):void{
			//trace('FV SETTING AUTO RESIZE = '+b);
			_autoResize = b;
			if(_visualization){
				//trace('FV SETTING AUTO RESIZE ON VIS ='+b);
				_visualization.autoResize = b;
				if(!b){ //If the vis isn't resizing to itself, then init to this size
					_visualization.height = height;
					_visualization.width = width;
					//update();
				}
			}
		}
		private var _autoResize:Boolean = true;		
		
		public function get continuousUpdates():Boolean{
			return _continuousUpdates;
		}
		public function set continuousUpdates(b:Boolean):void{
			_continuousUpdates = b;
			if(_visualization){
				_visualization.continuousUpdates = b;
			}
		}
		private var _continuousUpdates:Boolean = false;
		
		/**
		 * You can bind rawData to an array collection of data and this will then handle triggering
		 * buildData whenever something changes. 
		 * @param arr
		 * 
		 */		
		public function set rawData(arr:ArrayCollection):void{
			if(arr != _rawData){
				_rawData = arr;
				buildData(_rawData);
			}
		}
		
		private var _rawData:ArrayCollection;
						
		// ========================================
		// Public properties
		// ========================================	
		
		/**
		 * Flare Visualization instance.
		 */
		public function get visualization():Visualization {
			return _visualization;
		}
		
		public function set visualization(v:Visualization):void{
			_visualization = v;
			if(panZoomEnabled){				
				//Now we have a vis, we can set pan and Zoom again to set hit area
				panZoomEnabled = true;
			}
		}
		
		/** 
		 * 	The visualization operators used by this visualization. This
		 *  should be an array of IOperator instances. 
		 */
		public function set operators(a:Array):void {
			if(_visualization){
				_visualization.operators.list = a;
				//trace('OP UPDATE');
				//_visualization.update();
			}
			_operators = a;
		}
		
		private var _operators:Array;
		
		/** 
		 * 	The interactive controls used by this visualization. This
		 *  should be an array of IControl instances. 
		 */
		public function set controls(a:Array):void {
			if(_visualization){
				_visualization.controls.list = a;
				update();
			}
			_controls = a;
		}
		
		private var _controls:Array;				
		
		public function set layout(l:Layout):void{
			if(!_operators){
				_operators = [];
			}
			//First, remove old layout, if there is one
			for(var i:int=0;i<_operators.length;i++){				
				var op:Operator = _operators[i];
				if(op == _layout){
					delete _operators[i];
					break;
				}
			}
			_operators.push(l);
			operators = _operators;
			_layout = l;
			update(1);
		}
		
		private var _layout:Layout;
		
		
		/**
		 * For some visualisations you may wish to have a number of layouts that
		 * you can switch between for a given dataset. We could externally make 
		 * a new layout and set layout = it, but sometimes layouts also need other
		 * things to go with them (like a simulation for a force directed layout).
		 * SO.... you can build all your layouts ahead of time, store them here, then
		 * refer to them by name via the 'changeLayout(name)' function. You can manually
		 * manage this, or you can add and remove layouts to it via the addLayout(name,layout)
		 * and removeLayout(name) functions
		 */
		public var layouts:Object;
		
		
		/** 
		 * Returns the axes for the backing visualization instance. 
		 */
		public function get axes():Axes 			{ return _visualization.axes; }
		
		/** 
		 * Returns the CartesianAxes for the backing visualization instance. 
		 */
		public function get xyAxes():CartesianAxes 	{ return _visualization.xyAxes; }				
				
		/** 
		 *  Legacy support and wrapper to function set data(val) method 
		 */
		public function set dataSet(d:*):void 		{	this.data = d;		}

		[Bindable("dataChange")]
		/** 
		 *  Sets the data visualized by this instance. The input value can be
		 *  an array of data objects, a Data instance, or a DataSet instance.
		 *  Any existing data will be removed and new NodeMode instances will
		 *  be created for each object in the input arrary. 
		 */
		public function get data():Object {
			return _data;
		}
		
		public function set data( value:Object ):void {
			if ( _data != value ) {				
				_data = value;
				
				var d:Data = _data as Data;
				//trace('FLARE VIS SET DATA='+data);
				dataChanged = true;
				//trace('FLARE VIS INVALIDATE PROPS');
				invalidateProperties();											
				dispatchEvent( new FlexEvent( FlexEvent.DATA_CHANGE ) );
			}
		}				
		
		override public function set height(value:Number):void{
			super.height = value;
			if(!autoResize && _visualization){
				trace('FV SETTING VIS HEIGHT = '+value); 
				_visualization.height = value;				
			}			
			//trace('FLARE VIS height='+value);
		}
		
		override public function set width(value:Number):void{
			super.width = value;
			if(!autoResize && _visualization){	
				trace('FV SETTING VIS WIDTH = '+value); 
				_visualization.width = value;				
			}
			//trace('FLARE VIS width='+value);			
		}
				
		
		// ========================================
		// Protected properties
		// ========================================	
		
		/**
		 * Backing variable for <code>visualization</code> property.
		 */
		protected var _visualization:Visualization;
		
		/**
		 * Backing variable for <code>data</code> property.
		 */
		protected var _data:Object;
		
		/**
		 * Indicates the <code>data</code> property was invalidated.
		 */
		protected var dataChanged:Boolean;
		
		/**
		 * Mask for Flare Visualization.
		 */
		protected var visualizationMask:Sprite;	
		
		
		// ========================================
		// Constructor
		// ========================================	
		
		/**
		 * Constructor.
		 */
		public function FlareVisualization(data:Data=null)
		{
			super();
			
			if(!data){
				data = new Data();
			}			
			_data = data;
			
			//force setter:
			panZoomEnabled = _panZoomEnabled;
			
			visualization = new Visualization( _data as Data );		
			//This is just ridiculous overkill, so remove it
			visualization.removeEventListener( Event.RENDER, _visualization.setHitArea );
			visualization.x = xOffset;		
			visualization.y = yOffset;
			visualization.continuousUpdates = continuousUpdates;
			
			//trace('FV INIT SETTING AUTO RESIZE ON VIS ='+_autoResize);
			visualization.autoResize = this.autoResize;
			if(!autoResize){
				visualization.width = width;
				visualization.height = height;
			}
			
			BindingUtils.bindProperty(this, "width", _visualization, "width");
			BindingUtils.bindProperty(this, "height", _visualization, "height");
								
			//Set the controls and operators again now that we actually have a visualisation
			if(_controls){
				controls = _controls;
			}
			if(_operators){
				operators = _operators;
			}
		}							
		
		// ========================================
		// Public Methods
		// ========================================			
		/**
		 * This takes an array of objects and returns the Flare Data object
		 * it builds from it. You must override this in your extended class
		 * as the form of your data is usually quite unique. Example code 
		 * left commented out here for reference. You should also have this function
		 * set the data object.
		 * 
		 * <p>If your data format doesn't neatly sit into a single array (maybe you have nodes and 
		 * edges as two objects) then you can store them locally and then call buildData without passing
		 * it anything, and use your local vars instead.</p>
		 * 
		 * @param arr Input array
		 * @param fullRefresh can be used to alter the build process between a full build from scratch
		 * or an update build of data, preserving existing information.
		 * @return Flare Data object	
		 */		
		public function buildData(arr:ArrayCollection=null, fullRefresh:Boolean=true):Data{
			/*
			//Will probably move this to an 'createData' function that
			//passes all the configured vars so you don't have to think about
			//it... and will set up a root if your vis is of type 'tree'
			//bit more to add for all this to work.		
			var d:Data = new Data(false,nodeSkin,edgeSkin,'description');					
			var root:NodeMode = d.createRootNode();			
			root.data.type = 'root';
			root.data.description = 'The entire World!';
			
			
			//Keyed lookup for easy edge addition in step 2
			var nodeLookup:Object = {};
			var row:Object;
			var ns:NodeMode;				
			
			//Step 1: Add all rows of data;			
			for each(row in arr){
				ns = d.addNode(row);				
				nodeLookup[row.id] = ns; 
			}
			
			//Step 2: Add edges			
			for each(ns in d.nodes){
				if(ns.data.type == 'host'){
					d.addEdgeFor(root,ns);
				}
				if(ns.data.hasOwnProperty('host')){
					d.addEdgeFor(nodeLookup[ns.data.host],ns);
				}
				else if(ns.data.hasOwnProperty('dir')){
					d.addEdgeFor(nodeLookup[ns.data.dir],ns);
				}
			}					
			this.data = d;			
			return d;*/
			
			return null;
		}
		
		public function update(t:*=null):void{
			//trace('FLARE VIS UPDATE');
			if(!_visualization){
				return;
			}
			var trans:Transitioner = _visualization.update(t);
			if(trans){
				trans.play();
			}
		}
		
		/**
		 * Creates a new edge between the given nodes and adds it to the
		 * data collection, applying any styling to the edge defined in 'edgeStyling'
		 * @param d The data to source the nodes, and add this edge to
		 * @param source the source node (must already be in the data set)
		 * @param target the target node (must already be in the data set)
		 * @param directed indicates the directedness of the edge (null to
		 *  use this Data's default, true for directed, false for undirected)
		 * @param data a data tuple containing data values for the edge
		 *  instance. If non-null, this will become the EdgeMode's
		 *  <code>data</code> property.
		 * @return the newly added EdgeMode
		 */
		public function addEdgeFor(d:Data,source:NodeMode, target:NodeMode,
								   directed:Object=null, data:Object=null):EdgeMode{			
			var em:EdgeMode = d.addEdgeFor(source,target,directed,data);
			
		//	trace('ADD EDGE HAVE STYLE? '+edgeStyling);
			//Now apply styling if defined:
			if(edgeStyling){				
				var edgeStyle:Object;
				var attr:String;
				//Start with most generic styling and get more specific as we go down so that
				//most specific override least.
			//	trace('HAVE EDGESTYLING Looking for '+source.nodeType+' to '+target.nodeType);
				if(edgeStyling['*:*']){//Cover all styling					
					edgeStyle = edgeStyling['*:*'];
					for(attr in edgeStyle){
						em[attr] = edgeStyle[attr];
					}
				}		
				if(edgeStyling[source.nodeType+':*']){//From given node to any sort
					edgeStyle = edgeStyling[source.nodeType+':*'];
					for(attr in edgeStyle){
						em[attr] = edgeStyle[attr];
					}
				}
				if(edgeStyling['*:'+target.nodeType]){//From any to given to node
					edgeStyle = edgeStyling['*:'+target.nodeType];
					for(attr in edgeStyle){
						em[attr] = edgeStyle[attr];
					}
				}
				if(edgeStyling[source.nodeType+':'+target.nodeType]){ //Specific from and to
					//trace('FROM TO = '+source.nodeType+':'+target.nodeType);
					edgeStyle = edgeStyling[source.nodeType+':'+target.nodeType];
					for(attr in edgeStyle){
						//trace('FOUND SPECIFIC STYLE :'+attr+'='+edgeStyle[attr]);
						em[attr] = edgeStyle[attr];
					}
				}										
			}			 
			return em;
		}
		
		/**
		 * Reset the visualisation's zoom and pan values
		 * 
		 */
		public function reset():void{
			_visualization.reset();			
		}
		
		public function changeLayout(layoutName:String):void{		
			trace('CHANGE LAYOUT ['+layoutName+'] = '+layouts[layoutName]);
			if(layouts && layouts[layoutName]){
				layout = layouts[layoutName];
				update(1);
			}
		}
		
		public function addLayout(layoutName:String,layout:Layout):void{
			if(!layouts){
				layouts = {};				
			}
			layouts[layoutName] = layout;
		}
		
		public function removeLayout(layoutName:String):void{
			delete layouts[layoutName];
		}
		
		
		// ========================================
		// Protected methods
		// ========================================	
				
		
		/**
		 * @inheritDoc
		 */
		override protected function createChildren():void
		{	
			//trace('CREATE CHILDREN ON FLARVIS');
			addChild( _visualization );
			
			visualizationMask = new VisMask();			
			addChild( visualizationMask );
			
			_visualization.mask = visualizationMask;		
		}
		
		override protected function commitProperties():void
		{	
			//trace('Flare Visualization Commit properties');
			if ( dataChanged )
			{
				dataChanged = false;				
				if ( data == null )			_visualization.data = null;
				else if ( data is Data) 	_visualization.data = data as Data;
				else if ( data is Array )	_visualization.data = Data.fromArray( data as Array);
				else if ( data is DataSet )	_visualization.data = Data.fromDataSet( data as DataSet);
				else						throw new Error( "Unrecognized data set type: " + data );								
			}

			super.commitProperties();			
			
			//trace('COMMIT PROPERTIES UPDATE LATER');
			_visualization.updateLater();
		}
		
		/**
		 * @inheritDoc
		 */
		override protected function updateDisplayList( unscaledWidth:Number, unscaledHeight:Number ):void
		{			
			// draw 'hitbox'
			//trace('FlareVisualization updateDisplayList');
			graphics.clear();
			
			graphics.beginFill( 0xffffff, 0.0 );
			graphics.drawRect( 0, 0, unscaledWidth, unscaledHeight );
			graphics.endFill();
			
			// update visualization mask
			
			visualizationMask.graphics.clear();
			
			visualizationMask.graphics.beginFill( 0xff0000 );
			visualizationMask.graphics.drawRect( 0, 0, unscaledWidth, unscaledHeight );
			visualizationMask.graphics.endFill();
			
			//visualizationMask.cacheAsBitmap = true;
			_visualization.updateLater();
			//trace('VIS updateDisplayList update Display List');		
		}
		
			
	}
}
