package flare.vis.operator.layout
{
	import flare.util.Orientation;
	import flare.vis.data.Data;
	import flare.vis.data.EdgeMode;
	import flare.vis.data.NodeMode;

	public class RootlessNodeLinkLayout extends Layout
	{
		// -- Properties ------------------------------------------------------
		
		/** Property name for storing parameters for this layout. */
		public static const PARAMS:String = "rootlessNodeLinkLayout";
		
		public static const LEFTTORIGHT:String = "leftToRight";
		public static const TOPTOBOTTOM:String = "topToBottom";
		
		private var _orient:String = Orientation.LEFT_TO_RIGHT; // orientation
		private var _bspace:Number = 5;  // the spacing between sibling nodes
		private var _tspace:Number = 25; // the spacing between subtrees
		private var _dspace:Number = 50; // the spacing between depth levels		
		private var _maxDepth:int = 0;

		private var rootPositions:Array; 
		private var visitedNodes:Object;		
		private var orphanNodes:Array;
		private var highestDegreeNode:NodeMode;
		private var rootNodes:Array;
		
		//Padding
		public var yOffset:Number = 50;
		public var xOffset:Number = 50;
		
		/** The orientation of the layout. */
		public function get orientation():String { return _orient; }
		public function set orientation(o:String):void { _orient = o; }
		
		/** The space between successive depth levels of the tree. */
		public function get depthSpacing():Number { return _dspace; }
		public function set depthSpacing(s:Number):void { _dspace = s; }
		
		/** The space between siblings in the tree. */
		public function get breadthSpacing():Number { return _bspace; }
		public function set breadthSpacing(s:Number):void { _bspace = s; }
		
		/** The space between different sub-trees. */
		public function get subtreeSpacing():Number { return _tspace; }
		public function set subtreeSpacing(s:Number):void { _tspace = s; }
		
		public var childWiggle:Number;
				
		// -- Methods ---------------------------------------------------------		
		/**
		 * Creates a new RootlessNodeLinkLayout.
		 * @param orientation the orientation of the layout
		 * @param depthSpace the space between depth levels in the tree
		 * @param breadthSpace the space between siblings in the tree
		 * @param subtreeSpace the space between different sub-trees
		 * @param childWiggle the amount child nodes are shifted about their normal position to reduce line overlap 
		 * 					  between parent and child nodes (especially loop-back situation)
		 */		
		public function RootlessNodeLinkLayout(	orientation:String=Orientation.LEFT_TO_RIGHT, depthSpace:Number=20,
												breadthSpace:Number=50, subtreeSpace:Number=25,childWiggleVal:Number=25){
			_orient = orientation;
			_dspace = depthSpace;
			_bspace = breadthSpace;
			_tspace = subtreeSpace;
			childWiggle = childWiggleVal;
		}
		
		private function orderDegree(a:NodeMode, b:NodeMode):int 
		{ 
			return a.degree<b.degree?1:a.degree>b.degree?-1:0;			
		} 
		
		private function orderOutDegree(a:NodeMode, b:NodeMode):int 
		{ 
			return a.outDegree<b.outDegree?1:a.outDegree>b.outDegree?-1:0;			
		} 
		
		/** @inheritDoc */
		protected override function layout():void
		{
			updateEdgePoints(_t); 	// update edges
			width = xOffset;
			height = yOffset;						
			orphanNodes = [];
			visitedNodes = {};	
			rootNodes = [];
			var xPos:Number;
			var yPos:Number;
			
			//trace('============= LAYOUT START ============ HEIGHT IS: HEIGHT = '+height+' width='+width);				
			var nm:NodeMode;
			var d:Data = visualization.data as Data;
			if(d.nodes.length == 0){
				return;
			}
			//Apply depth values:
			for each(nm in d.nodes){
				nm.assignDepths(0);
				
				//Root nodes are those with outgoing links...//no incoming
				if(nm.outDegree > 0){//(nm.inDegree == 0 && 
					rootNodes.push(nm);
				}
			}			
			if(rootNodes.length > 0){
				//Order such that nodes with most links are first.
				rootNodes.sort(orderOutDegree);
				
				//START POSITIONS
				if(orientation == LEFTTORIGHT){
					yPos = yOffset+((nm.outDegree-1)*(nm.height/2));
					xPos = xOffset;
				}
				else{
					xPos = xOffset+((nm.outDegree-1)*(nm.width/2));
					yPos = yOffset;
				}
				//Init higestDegree with the one we're visiting
				highestDegreeNode = rootNodes[length-1];
				
			
				//Lay out Root Nodes
				for(var i:int=0;i<rootNodes.length;i++){
					//trace('PLACING NODE WITH OD='+nm.outDegree);
					nm = rootNodes[i];
					if(!visitedNodes[nm.data.uid]){				
		
						if(orientation == LEFTTORIGHT){					
							yPos = height+((nm.outDegree)*(nm.height/2));						
							yPos+=subtreeSpacing;						
						}
						else{				
							xPos = width+((nm.outDegree)*(nm.width/2));						
							xPos+=subtreeSpacing;						
						}
						placeNode(nm,xPos,yPos);
					}							
				}			
			}
			
			//Now lay out remainding nodes
			if(orientation == LEFTTORIGHT){
				yPos = height+subtreeSpacing;
				xPos = xOffset;
			}
			else{
				yPos = yOffset;
				xPos = width+subtreeSpacing;
			}
						
			var lastSize:Number = 0;
			
			for each(nm in d.nodes){
				if(!visitedNodes[nm.data.uid]){	
					if(orientation == LEFTTORIGHT){															
						xPos+= subtreeSpacing +lastSize;		
						lastSize = nm.width;
					}
					else{														
						yPos+=subtreeSpacing+lastSize;
						lastSize = nm.height;
					}
					placeNode(nm,xPos,yPos);
				}
			}
								
			updateEdgePoints(_t); 	// update edges
			super.layout(); 		//Set parent size			
			//trace('============= LAYOUT END ============ HEIGHT IS: HEIGHT = '+height+' width='+width);
		}					
					
		protected function placeNode(n:NodeMode,nodeX:Number,nodeY:Number):void
		{	
			var i:int;
			//These layouts can include nodes more than once. If we've already been here once
			//then back away quietly.
			if(visitedNodes[n.data.uid]){
				//trace('PLACED '+n.data.name+' already');
				return;
			}
			else{			
				visitedNodes[n.data.uid] = n;			
			}
				
			
			// NOW, we are going to visit the edges of this node and change the type of edge based on 
			//whether we think there's going to be overlap.			
			/*n.visitEdges(function(e:EdgeMode):void {				
			
			}, NodeMode.OUT_LINKS);
			n.visitEdges(function(e:EdgeMode):void {				
							
			}, NodeMode.IN_LINKS);
			*/
			//Keep track of which node has the most incoming/outgoing edges.
			if(n.degree > highestDegreeNode.degree){
				highestDegreeNode = n;
			}
			//Setting values by way of a transitioner, for animation purposes
			//trace('placeNode: '+n.data.name+'['+n.data.objectId+']');
			var o:Object = _t.$(n);
			
			n.h = _t.endSize(n, _rect).height;
			
			o.x = nodeX;
			o.y = nodeY;
								
			// update layout size					
			if((o.x+n.width) > width){
				width = o.x+n.width;
			}
			if((o.y+n.height) > height){
				height = o.y+n.height;
			}

			//trace('--->'+n.data.name+'['+n.data.objectId+'] x='+o.x+' y='+o.y+' width='+width+' height='+height+' =-=-=-=-=-=-=-= My Children number '+n.outNodes.length)				
			
			//Traverse children
			var on:Array = n.outNodes;
			var cn:NodeMode;			
			var childX:Number;
			var childY:Number;
			//Child wiggle is to stagger the placement of subsequent child nodes such that lines between them 
			//are less likely to overlap and be obscured. Only start after root and first level
			var childWiggleAmt:Number = n.nodeDepth<2?0:n.nodeDepth%2==1?childWiggle:-childWiggle;
			
			if(on.length > 0){
				if(orientation == LEFTTORIGHT){
					childX =o.x+(n.width*2)+depthSpacing;
					childY = o.y;			
					for(i=0;i<on.length;i++){						
						cn =on[i];												
						//trace('LEFT RIGHT '+cn.data.name+' X='+childX+' Y='+((childY - ((on.length-1)*(n.height/2) )+(i*breadthSpacing))+childWiggleAmt));
					//	trace('NODE SIZE = '+n.width+' : '+n.height);
						placeNode(cn,childX,(childY - ((on.length-1)*(n.height/2) )+(i*breadthSpacing))+childWiggleAmt);
					}									
				}
				if(orientation == TOPTOBOTTOM){
					childX = o.x;
					childY = o.y+depthSpacing+(n.height);//+(n.height*2)
					for(i=0;i<on.length;i++){		
						cn =on[i];
						//trace('TOP BOTTM ['+i+']'+n.data.name+'['+n.data.objectId+'] Placing =-=-= |'+cn.data.name+':'+cn.data.objectId);
						placeNode(cn,(childX - ((on.length-1)*(cn.width/2) )+(i*breadthSpacing))-childWiggleAmt,childY);
					}				
				}
			}
		}
		
		private function setVisibility(n:NodeMode, o:Object, visible:Boolean):void
		{
			o.alpha = visible ? 1.0 : 0.0;
			o.mouseEnabled = visible;
			if (n.parentEdge != null) {
				o = _t.$(n.parentEdge);
				o.alpha = visible ? 1.0 : 0.0;
				o.mouseEnabled = visible;
			}			
		}
		
		private function setSizes(n:NodeMode):void
		{
			_t.endSize(n, _rect);
			n.w = _rect.width;
			n.h = _rect.height;
		}
		
		private function spacing(l:NodeMode, r:NodeMode, siblings:Boolean):Number
		{
			var w:Boolean = Orientation.isVertical(_orient);
			return (siblings ? _bspace : _tspace) + 0.5 *
				(w ? l.w + r.w : l.h + r.h)
		}		
		// -- Parameter Access ------------------------------------------------
		
		private function params(n:NodeMode):Params
		{
			var p:Params = n.props[PARAMS] as Params;
			if (p == null) {
				p = new Params();
				n.props[PARAMS] = p;
			}
			if (p.number == -2) { p.init(n); }
			return p;
		}		
	} // end of class NodeLinkTreeLayout	
}


import flare.vis.data.NodeMode;

class Params {
	public var prelim:Number = 0;
	public var mod:Number = 0;
	public var shift:Number = 0;
	public var change:Number = 0;
	public var number:int = -2;
	public var ancestor:NodeMode = null;
	public var thread:NodeMode = null;
	
	public function init(item:NodeMode):void
	{
		ancestor = item;
		number = -1;
	}
	
	public function clear():void
	{
		number = -2;
		prelim = mod = shift = change = 0;
		ancestor = thread = null;
	}
} // end of class Params